/*
	$Id: resource_manager.h,v 1.8 2001/04/11 19:56:14 japj Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! clanCore="Resources"

#ifndef header_resource_manager
#define header_resource_manager

#include <list>
#include <string>
class CL_Resource;
class CL_InputSourceProvider;

class CL_ResourceManager
//: The ClanLib resource manager.
// The resource manager is used to retrieve resources from a given resource source.
// This can either be a resource script file (used as input to the
// datafile compiler), or a datafile with all the resources included into
// one large gzipped resource file.
// <br>
// To speedup loading of resources in a game, you can load entire sections
// of resources at once. When a resource in the section is requested, it is
// returned instantly without having to access the disk. This is especially
// useful to make sure all the game resources are loaded before the game is
// started.
// <br>
// Resources are normally not retrieved using the get_resource() function.
// Instead, you should load the resource using the appropiate resource type
// class. For instance, a surface is easiest loaded like this:
// <code>
// CL_ResourceManager res_manager("my_datafile.dat", true);
// CL_Surface my_surface("my_surface", res_manager);
// </code>
// <br>
// Getting the same resource twice won't create a new instance of the
// resource; they are reference counted.
{
//!Construction:
public:
	CL_ResourceManager(
		const std::string &config_file,
		CL_InputSourceProvider *provider = NULL,
		bool read_directly_from_source=false,
		bool delete_inputsource_provider=false);
	//: Resource Manager constructor.
	//!param: config_file - the name of the file in which the resources are defined
	//!param: provider - the optional inputprovider in which, the resource file is stored
	//!param: read_directly_from_source - if true, any resources are loaded directly from their source, meaning that any datafile-directive is ignored.

	CL_ResourceManager(
		const std::string &config_file,
		const bool is_datafile );
	//: Resource Manager constructor.
	//!param: file_name - the name of the file to open
	//!param: is_datafile - indicates if the file is a scriptfile or a datafile

	CL_ResourceManager(const CL_ResourceManager &copy);

	CL_ResourceManager();

	~CL_ResourceManager();

//!Attributes:
public:
	CL_Resource &get_resource(const std::string &res_id);
	//: Returns a pointer to the CL_Resource representing the given resource

	std::list<std::string> *get_all_resources();
	//: Returns a list of all resources available. Primarily used by the datafile compiler to 
	//: build datafiles from resources.
	//!retval: The list of resources available. You'll have to delete the list returned.

	std::list<std::string> *get_resources_of_type(const std::string &type_id);
	//: Returns a list of all resources available matching a given type. 
	//: Primarily used by the ClanCompiler to build datafiles from resources.
	//!retval: The list of resources available. You'll have to delete the list returned.

	CL_InputSourceProvider *get_resource_provider() const;
	//: Returns a pointer to the inputsourceprovider, in which all resources are stored
	//: (this can be a fileprovider or a datafileprovider depending on method used to load the script file)
	//!retval: Pointer to inputsourceprovider containing resource data.

	bool is_from_source();
	//: Returns true if the resources are loaded from source. False if loaded
	//: from a datafile.

//!Operations:
public:
	void operator = (const CL_ResourceManager &copy);
	// Copy a resource manager.

	void load_all();
	//: Loads all resources into memory.

	void unload_all();
	//: Unloads all resources from memory.

	void load_section(const std::string &section_name);
	//: Loads all resources in a given section into memory.

	void unload_section(const std::string &section_name);
	//: Unloads all resources in a given section into memory.

//!Implementation:
public:
	CL_ResourceManager(class CL_ResourceManager_File *impl);
	class CL_ResourceManager_File *impl;
};
#endif
