/*
	$Id: font.h,v 1.2 2001/04/13 22:40:18 plasmoid Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------

	File purpose:
		Simple font support.
*/

//! clanDisplay="Fonts"

#ifndef header_font
#define header_font

class CL_Target;
class CL_ResourceManager;
class CL_Font_Description;
class CL_Font_Generic;

class CL_Font
//: ClanLib's font class.
// <p>This class allows you to easilly draw text onto the backbuffer or to a
// renderer target.</p>
//
// <p>The font can be created in two ways. Either it is loaded from a
// resource, or it is created using the font description interface. The font
// and its description have the same relationship/functionality as the <a
// class="CL_Surface">surface</a> and its <a
// class="CL_SurfaceProvider">provider</a>.</p>
//
// <p>Please read the CL_Font_Description page if you want an description on
// how to create your own font.</p>
{
public:
	enum alignments
	{
		ALIGN_LEFT,
		ALIGN_CENTER,
		ALIGN_RIGHT
	};
	// Contains the alignment of the text.
	// Doesn't need to be set and is used to hold and set the current alignment
	// You can safely ignore this
	//!param: ALIGN_LEFT - Aligns the text left.
	//!param: ALIGN_CENTER - Aligns the text centre.
	//!param: ALIGN_RIGHT - Aligns the text right.

public:
//!static_init:
	static CL_Font *create(CL_Font_Description *font_desc);
	// Creates the font from a font description.
	//!param: font_desc - The font description used to construct the font.
	
	static CL_Font *load(
		const char *resource_id,
		CL_ResourceManager *resource_manager);
	// Loads the font from resources.
	//!param: resource_id - The resource ID of the font.
	//!param: resource_manager - The resource manager used to load the
	//!param:                    resources from.

//!construction:
	CL_Font(CL_Font_Description *provider);
	//: Creates a font from a font provider/description

	CL_Font(const char *resource_id, CL_ResourceManager *manager);
	//: Loads a font from a resource file.

	CL_Font(const CL_Font &font);
	//: Make a copy of an other font, but share the image data.

 // CL_Font(const CL_Font_Generic &font);
  //: Make a font

	virtual ~CL_Font();

//!functions:
	int get_height();
	//: Returns the height of the font.
	//!retval: Width of the font.

	int get_text_width(const char *text);
	//: Return the width of 'text' if drawn with this font.
	//!param: text - Text string the width should be calculated from.
	//!retval: Width of the text (in pixels).

	int get_char_width(const char character);
	//: Return the width of character if drawn with this font.
	//!param: text - Character the width should be calculated from.
	//!retval: Width of the character (in pixels).
	
	void print_left(int x, int y, const char *text, int n_height=1);
	//: Prints 'text' left aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is drawn at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: text - The text to be drawn with this font.
	//!param: n_height - Adds font height + n_height pixels to Y coordinate on new line ('\n').

	void print_left(int x, int y, float scale_x, float scale_y, const char *text);
	//: Prints 'text' left aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is drawn at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: scale_x - Scale font width scale_x times.
	//!param: scale_y - Scale font height scale_y times.
	//!param: text - The text to be drawn with this font.
	
	void print_center(int x, int y, const char *text);
	//: Prints 'text' center aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is centered at.
	//!param: y - Y coordiante (in pixels) the text is centered at.
	//!param: text - The text to be drawn with this font.

	void print_right(int x, int y, const char *text);
	//: Prints 'text' right aligned at coordinates ('x','y').
	//!param: x - X coordinate (in pixels) the text is ends at.
	//!param: y - Y coordiante (in pixels) the text is drawn at.
	//!param: text - The text to be drawn with this font.
	
	void put_target(int x, int y, const char *text, CL_Target *target, int alignment = ALIGN_LEFT);
	//: Prints 'text' aligned with alignment at coordinates ('x','y') to the target surface.
	//: If CL_Font::ALIGN_LEFT is specified the text is draw starting from x, y.
	//: If CL_Font::ALIGN_CENTER is specified the text is centered at x, y.
	//: If CL_Font::ALIGN_RIGHT is specified the text ends at x, y;
	//!param: x - X coordinate (in pixels)
	//!param: y - Y coordiante (in pixels)
	//!param: text - The text to be drawn with this font.
	//!param: target - the target surface to blit to.
	//!param: alignment - the alignment of the text, CL_Font::ALIGN_LEFT, CL_Font::ALIGN_CENTER, CL_Font::ALIGN_RIGHT

  int change_size(int size);
  //: Changes the size of the font used.
  //: This function only does something meaninful if you use TrueType fonts
  //!param: size - The new size to use.
  //!retval:  Returns the size set or -1 if the font size cannot be changed to size

  unsigned int change_colour(unsigned char r, unsigned char g, unsigned char b, unsigned char a);
  //: Changes the colour of the font used.
  //!param: r - The red component
  //!param: g - The green component
  //!param: b - The blue component
  //!param: a - The alpha component
  //!retval: Returns the font set in packed ARGB format.


//!implementation:
	CL_Font(class CL_Font_Generic *impl);
	class CL_Font_Generic *impl;
};

#endif
