/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Framework"

#ifndef header_component
#define header_component

#include <string>

#include "../signals.h"
#include "../Core/Math/rect.h"
#include "../Core/Math/point.h"
#include "../Display/Input/key.h"

class CL_GUIManager;
class CL_ComponentStyle;
class CL_InputDevice;
class CL_ComponentOptions;
class CL_StyleManager;
class CL_Component_Generic;

class CL_Component
//: Generic component interface
{
//!Construction:
public:
	CL_Component(
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_Component(
		const CL_Rect &pos,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_Component(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);
	
	virtual ~CL_Component();

//!Attributes:
public:
	CL_Component *get_parent() const;
	// Returns the parent component, or NULL if none.

	CL_Component *get_root_parent();
	// Returns the root parent.

	CL_GUIManager *get_gui_manager() const;
	// Returns the GUI manager that this component is attached to, or NULL if none.

	CL_StyleManager *get_style_manager() const;
	// Returns the style manager used by this component.

	int get_width() const;
	// Returns the width of the component.

	int get_height() const;
	// Returns the height of the component.

	CL_Rect get_screen_rect() const;
	// Returns the components position in screen coordinates.

	const CL_Rect &get_position() const;
	// Returns the position of this component.

	bool has_child(CL_Component *component) const;
	// Returns true if the passed component is a child of this component.

	std::list<CL_Component *> &get_children();
	// Returns the list of child components.

	const std::list<CL_Component *> &get_children() const;
	// Returns the list of child components.

	CL_Rect get_children_rect();
	// Returns the bounding rectangle of the component's children.

	CL_Component *get_component_at(int pos_x, int pos_y);
	// Returns the component at (pos_x, pos_y).
	// NULL is returned if the point is outside the component.
	// The component itself is returned if the component has no child at the specified point.

	bool is_visible() const;
	// Returns true if component is visible.

	bool is_enabled() const;
	// Returns true if component accepts user input.

	bool is_popup() const;
	// Returns true if component is a popup window.

	bool has_focus() const;
	// Returns true if component has the focus.

	bool is_mouse_captured() const;
	// Returns true if component has mouse captured.

	bool is_removeflag_set() const;
	// I have no idea what this does... -- mbn 3. okt 2000.

//!Operations:
public:
	void update();
	// Calls sig_begin_paint(), then sig_paint(), and finally sig_end_paint().

	void set_position(const CL_Rect &new_pos);
	// Sets the position (relative to its parent) and size of this component.

	void set_position(int new_x, int new_y);
	// Sets the position of this component (relative to its parent).

	void set_size(int new_width, int new_height);
	// Sets the size of this component.

	void set_parent(CL_Component *parent);
	// Sets the parent of this component.

	void set_gui_manager(CL_GUIManager *gui);
	// Sets the GUI manager this component is attached to.

	void set_width(int width);
	// Changes the width of the component.

	void set_height(int height);
	// Changes the height of the component.

	void show(bool show = true);
	// Changes the visibility flag.

	void enable(bool enable = true);
	// Enable/Disable user input.

	void popup(bool popup);
	// Change popup flag.

	void set_focus();
	// Give focus to this component.

	void capture_mouse();
	// Capture the mouse on the currently attached GUI manager. All mouse input
	// will be directed to this component until release_mouse() is called.

	void release_mouse();
	// Releases the mouse capture.

	void close();
	// Closes this component. What does that mean?? -- mbn 3. okt. 2000

	void run(CL_Component *parent = NULL);
	// Run the component as a modal component until it returns

	void quit();
	// Quit the modal loop of the component

	void clear_removeflag();
	// Clears the remove flag... dunno what the remove flag means. -- mbn 3. okt 2000

	void add_child(CL_Component *child, bool delete_child = false);
	// Adds the specified component as a children to this component.
	// If 'delete_child' is true, the child will be deleted when the component is deleted.

	void remove_child(CL_Component *child);
	// Removes the specified child from the component.

	void add_components(class CL_ComponentManager *component_manager);
	// Add all components in a component manager.

	void attach_style(CL_ComponentStyle *style);
	// Attaches a component style to the component.

	void detach_style(CL_ComponentStyle *style);
	// Removes a component style to the component.

//!Signals:
public:
	CL_Signal_v0 &sig_paint();
	// Draws the component. Must be called in a
	// begin_paint() / end_paint() session.

	CL_Signal_v0 &sig_begin_paint();
	// Called prior to component drawing. Sets up a cliprect and translocates
	// coordinates accordingly to this component.

	CL_Signal_v0 &sig_end_paint();
	// Cleans up when component drawing is finished (cleans up cliprects and
	// translocation stuff).

	CL_Signal_v3<CL_Component *, CL_InputDevice *, CL_Key> &sig_key_down();
	// Called when a key is pressed.

	CL_Signal_v3<CL_Component *, CL_InputDevice *, CL_Key> &sig_key_up();
	// Called when a key is released.

	CL_Signal_v4<CL_Component *, CL_InputDevice *, int, int> &sig_mouse_moved();
	// Called when the mouse is moved.

	CL_Signal_v2<int, int> &sig_mouse_down();
	// Called when the mousebutton is clicked.

	CL_Signal_v2<int, int> &sig_mouse_up();
	// Called when the mousebutton is released.

	CL_Signal_v0 &sig_mouse_entered();
	// Called when mouse enters the component area.

	CL_Signal_v0 &sig_mouse_left();
	// Called when mouse leaves the component area.

	CL_Signal_v0 &sig_got_focus();
	// Called when component gains focus.

	CL_Signal_v0 &sig_lost_focus();
	// Called when component lose focus.

	CL_Signal_v2<int, int> &sig_resized();
	// Called when component is resized.

	CL_Signal_v1<CL_Component *> &sig_child_add();
	// Called when a child component is added to the component.

	CL_Signal_v1<CL_Component *> &sig_child_remove();
	// Called when a child component is removed from the component.

	CL_Signal_v2<int &, int &> &sig_transform_coords();
	// Called when needing to convert parent coordinates into coordinates used by this component.
	// This is used in 3D guis to convert from screen coordinates to gui coordinates.
	// (project the screen cordinates down on the plane that the gui is laying on)
	
//!Implementation:
private:
	class CL_Component_Generic *impl;
};

#endif
