/*
	ClanGUI, copyrights by various people. Have a look in the CREDITS file.
	
	This sourcecode is distributed using the Library GNU Public Licence,
	version 2 or (at your option) any later version. Please read LICENSE
	for details.
*/

//! clanGUI="Controls"

#ifndef header_inputbox
#define header_inputbox

#include "component.h"

class CL_InputBox : public CL_Component
//: Inputbox component
{
//!Construction:
public:
	CL_InputBox(
		const CL_ComponentOptions &options,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	CL_InputBox(
		const CL_Rect &pos,
		const std::string &text,
		CL_Component *parent,
		CL_StyleManager *style = NULL);

	virtual ~CL_InputBox();

//!Attributes:
public:
	const std::string &get_text() const;
	//: Returns the inputfield text.

	int get_length() const;
	//: Returns the length of the inputfield text.

	const std::string &get_marked_text() const;
	//: Returns the marked text from the inputfield.

	bool has_marked_text() const;
	//: Returns true if part of the text has been marked by the user.

	int get_selection_start() const;
	//: Returns the position where the selection starts.

	int get_selection_length() const;
	//: Returns the length of current selection.

	int get_max_length() const;
	//: Returns the maximum permitted length of the text.

	bool in_password_mode() const;
	//: Returns true if password mode is enabled, false othewise.

	bool is_read_only() const;
	//: Returns whether the inputfield is read-only. 

	int get_cursor_position() const;
	//: Returns the current cursor position.

	bool is_edited() const;
	//: Returns true if the contents has been edited.

//!Operations:
public:
	void set_text(const std::string &text);
	//: Sets the line edit text to text, clears the selection and moves the cursor to the end of the line.
	// If necessary the text is truncated to get_max_length().

	void set_max_length(int length);
	//: Set the maximum length of the text in the editor.

	void set_password_mode(bool enable = true);
	//: If enable is true, display asterisks instead of the characters actually entered.

	void set_read_only(bool enable = true);
	//: Enables or disables read-only mode.
	
	void select_all();
	//: Selects all text, and moves the cursor to the end.

	void deselect();
	//: Deselects all text (i.e. removes marking) and leaves the cursor at the current position. 

	void set_selection(int start, int length);
	//: Sets the marked area at start and be length characters long. 

	void set_cursor_position(int pos);
	//: Sets the cursor position to pos. 

	void clear();
	//: Syntactic sugar for set_text("").

	void backspace();
	//: Deletes the character to the left of the text cursor and moves the cursor one position to the left.
	// If a text has been marked, the cursor will be put at the beginning of the marked text and the marked text will be removed.

	void del();
	//: Deletes the character on the right side of the text cursor.
	// If a text has been marked, the cursor will be put at the beginning of the marked text and the marked text will be removed.

	void cut();
	//: Cut the selected text if any.

	void move_cursor(int delta, bool mark = false);
	//: Moves the cursor position.
	// If delta is positive, the cursor is moved forward.
	// If delta is negative, the cursor is moved backward.
	// If mark is true, the text is marked.

	void move_cursor_word(int delta, bool mark = false);
	//: Moves the cursor by word.
	// If delta is positive, the cursor is moved forward.
	// If delta is negative, the cursor is moved backward.
	// If mark is true, the text is marked.

	void home(bool mark = false);
	//: Moves the text cursor to the left end of the line.
	// If mark is true, text will be marked towards the first position
	// If mark is false, any marked text will be unmarked if the cursor is moved. 

	void end(bool mark = false);
	//: Moves the text cursor to the right end of the line.
	// If mark is true, text will be marked towards the last position
	// If mark is false, any marked text will be unmarked if the cursor is moved. 

	void set_edited(bool on = true);
	//: Sets the edited flag to on. 
	// The edited flag is changed to true whenever the user changes its contents. 

//!Signals:
public:
	CL_Signal_v1<const std::string &> &sig_changed();
	//: This signal is emitted whenever text is changed in the inputfield.
	//!param: const std::string & - Contains the new string.

	CL_Signal_v0 &sig_return_pressed();
	//: This signal is emitted when enter is pressed in the inputfield.

//!Implementation:
private:
	CL_InputBox(const CL_InputBox &copy) : CL_Component(NULL, NULL) { return; } // disallow copy construction.
	class CL_InputBox_Generic *impl;
};

#endif
