/*
	$Id: netsession.h,v 1.8 2001/03/28 07:45:41 mbn Exp $

	------------------------------------------------------------------------
	ClanLib, the platform independent game SDK.

	This library is distributed under the GNU LIBRARY GENERAL PUBLIC LICENSE
	version 2. See COPYING for details.

	For a total list of contributers see CREDITS.

	------------------------------------------------------------------------
*/

//! clanNetwork="NetSessions"

#ifndef header_netsession
#define header_netsession

#include <string>

#define ACCESS_CHANNEL_READ 1
#define ACCESS_CHANNEL_WRITE 2

class CL_NetComputer;
class CL_NetGroup;
class CL_NetMessage;
class CL_ServerInfo;
class CL_EventTrigger;

class CL_NetSession
//: CL_NetSession is the interface to communcation with a network session/game.
{
//!Construction:
public:
	CL_NetSession(
		const std::string &app_id,
		int port);
	//: Create a netsession server on the specified port. Only netsession clients
	//: with same application ID will be able to connect to the server.

	CL_NetSession(
		const std::string &app_id,
		const std::string &hostname,
		int port);
	//: Create a netsession client and connect to a server.

	CL_NetSession(
		const CL_ServerInfo &server);
	//: Connect to a server coming from a server info structure. Usually used together
	//: with master browse servers and LAN broadcast browsing.

	CL_NetSession(const CL_NetSession &copy);
	//: Copy constructor.

	virtual ~CL_NetSession();

//!Attributes:
public:
	CL_EventTrigger *get_trigger();
	// Event trigger that triggers when the netsession object changes (on incoming data).

	CL_NetComputer &get_server();
	//: The server of the network game.

	CL_NetGroup &get_all();
	//: A group of all the computers in the netsession.
	
	bool peek(int channel) const;
	//: Returns true if a message is available on the specified channel.

	int access_status(int channel) const;
	//: Returns the access status flags on the channel.

	bool is_writable(int channel) const;
	//: Returns true if the ACCESS_CHANNEL_WRITE flag is set on channel.

	bool is_readable(int channel) const;
	//: Returns true if the ACCESS_CHANNEL_READ flag is set on channel.

//!Operations:
public:
	void operator =(const CL_NetSession &session);
	// Makes a copy of the current session. Since sessions are reference counted,
	// this just means you will have two handles for the same session.

	CL_NetMessage receive(int channel, int timeout = -1);
	//: Receives a message from the specified channel. The call is blocking, so
	//: don't call it if you aren't completly sure it will arrive (unless you set
	//: a timeout).
	//!param: channel - Channel to look for a message.
	//!param: timeout - Timeout, -1 means block forever.
	//!retval: The message received from the specified channel.

	void send(
		const int dest_channel,
		const CL_NetGroup &dest,
		const CL_NetMessage &message,
		bool reliable = true);
	//: Sends a message to a group of computers at netchannel 'dest_channel'.
	//!param: dest - The group of computers to receive the message.
	//!param: dest_channel - Netchannel the message is sent to.
	//!param: message - Data in the message sent. All the data from 'message' is duplicated before this function returns.
	//!param: reliable - Arrival reliability. True guarantees arrival (use TCP/Pony express), false doesn't (use UDP/US Mail).

	CL_NetComputer receive_computer_leave();
	//: Returns the first (if more) computer that has left the game, or NULL if
	//: none. The next time the function is called, it returns the next computer,
	//: etc.<br>
	//: CL_NetComputer instances are reference counted. A computer wont be forgotten
	//: until all instances of it has been destroyed. If a computer rejoins the server
	//: while there still exist a netcomputer instance, it will be recognized and
	//: cause a 'rejoin' event instead of a normal join event.

	CL_NetComputer receive_computer_join();
	//: Returns the first (if more) computer that is joining the game, or throws an exception
	//: if none joined. The next time the function is called, it returns the next computer,
	//: etc.

	CL_NetComputer receive_computer_rejoin();
	//: Returns the first (if more) computer that is re-joining the game, or throws an exception
	//: if none joined. The next time the function is called, it returns the next computer,
	//: etc. <br>
	//: A re-joining computer is simply a computer joining the game, that still
	//: havn't had its CL_NetComputer reference deleted. This provides a way to
	//: recognize a computer that was in the game.

	bool receive_session_closed();
	// Returns true if the session was closed.

	int receive_access_changed();
	//: Returns the first netchannel that has had its access changed. Returns -1 when no
	//: more channels have had their access changed.
	//: This function only works on a client netsession.

	void set_access(
		int channel,
		const CL_NetGroup &group,
		int access_rights = ACCESS_CHANNEL_READ | ACCESS_CHANNEL_WRITE);
	//: Sets the channel access flags on the channel for the computers specified.
	//: This function only has effect on a server netsession.

//!Implementation:
public:
	CL_NetSession(class CL_NetSession_Generic *impl);
	class CL_NetSession_Generic *impl;
};

#endif
