
#ifndef header_socket
#define header_socket

//! clanNetwork="Sockets"

#include "../signals.h"
#include "ip_address.h"

class CL_EventTrigger;

class CL_Socket
//: This is the lowlevel raw socket interface in clanlib.
// Its main purpose is to write the C sockets API, providing error handling 
// via exceptions, and socket waiting (select) via the clanlib event waiting 
// system (CL_EventListener, CL_EventTrigger).
{
public:
	enum Type
	{
		tcp,
		udp
	};

	enum ShutdownHow
	{
		shutdown_receive,
		shutdown_send
	};

//!Construction:
public:
	CL_Socket(int socket);
	//!construction:
	// Constructs an attached socket.

	CL_Socket(Type type);
	//!construction:
	// Constructs a new socket using the specified protocol type.

	CL_Socket(const CL_Socket &copy);
	//!construction:
	// Copy constructor.

	virtual ~CL_Socket();

//!Attributes:
public:
	int get_socket() const;
	//!attributes:
	// Returns the raw socket handle (for use with the lowlevel socket functions).

	CL_EventTrigger *get_read_trigger() const;
	//!attributes:
	// Returns an event trigger that triggers when socket has data ready to be read.

	CL_EventTrigger *get_write_trigger() const;
	//!attributes:
	// Returns an event trigger that triggers when socket is ready for additional data.

	CL_EventTrigger *get_exception_trigger() const;
	//!attributes:
	// Returns an event trigger that triggers when an exception occours on the socket.

	CL_IPAddress get_source_address() const;
	//!attributes:
	// Returns the socket name.

	CL_IPAddress get_dest_address() const;
	//!attributes:
	// Returns the name of the destination socket we are connected to.

	CL_Signal_v0 &sig_read_triggered();
	//!attributes:
	// This signal is invoked when the socket has data ready to be read.

	CL_Signal_v0 &sig_write_triggered();
	//!attributes:
	// This signal is invoked when the socket has data ready to be written.

	CL_Signal_v0 &sig_exception_triggered();
	//!attributes:
	// This signal is invoked when an exception occoured on the socket.

//!Operations:
public:
	void set_nonblocking(bool nonblocking = true);
	//!operations:
	// Sets the socket blocking mode.

	void set_nodelay(bool nodelay = true);
	//!operations:
	// If enabled, don't delay send to coalesce packets.

	int send(const void *data, int size);
	//!operations:
	// Writes data to socket. Returns the amount that was written.

	int send(const void *data, int size, const CL_IPAddress &dest);
	//!operations:
	// Writes data to socket, using the specified destination host.

	int recv(void *data, int size);
	//!operations:
	// Reads data from the socket. Returns the amount that was read.

	int recv(void *data, int size, CL_IPAddress &from);
	//!operations:
	// Reads data from the socket, storing the from address in the passed parameter.

	void connect(const CL_IPAddress &address);
	//!operations:
	// Initiate a connection on the socket.

	void shutdown(ShutdownHow how);
	//!operations:
	// Shut down part of full-duplex connection.

	void bind(const CL_IPAddress &address);
	//!operations:
	// Bind the socket to the specified address.

	void listen(int backlog);
	//!operations:
	// Listen for connections on the socket.

	CL_Socket accept();
	//!operations:
	// Accept a connection on the socket.

//!Implementation:
public:
	CL_Socket(class CL_Socket_Generic *impl);
	class CL_Socket_Generic *impl;
};

#endif
